=== modified file 'bzrlib/diff.py'
--- bzrlib/diff.py	2007-11-21 14:08:53 +0000
+++ bzrlib/diff.py	2007-11-21 14:55:46 +0000
@@ -528,3 +528,35 @@
         else:
             old_entry.diff(self.text_diff, old_name, self.old_tree, new_name,
                            new_entry, self.new_tree, self.to_file)
+
+    def diff_symlink(self, old_target, new_target):
+        if old_target is None:
+            self.to_file.write('=== target is %r\n' % new_target)
+        elif new_target is None:
+            self.to_file.write('=== target was %r\n' % old_target)
+        else:
+            self.to_file.write('=== target changed %r => %r\n' %
+                               (old_target, new_target))
+
+    def diff_text(self, from_file_id, to_file_id, from_label, to_label):
+        """Diff the content of a given file in two trees
+        
+        :param from_file_id: The id of the file in the from tree.  If None,
+            the file is not present in the from tree:
+        :param to_file_id: The id of the file in the to tree.  If None,
+            the file is not present in the to tree:
+        """
+        def _get_text(tree, file_id):
+            if file_id is not None:
+                return tree.get_file(file_id).readlines()
+            else:
+                return []
+        try:
+            from_text = _get_text(self.old_tree, from_file_id)
+            to_text = _get_text(self.new_tree, to_file_id)
+            self.text_diff(from_label, from_text, to_label, to_text,
+                           self.to_file)
+        except errors.BinaryFile:
+            self.to_file.write(
+                  ("Binary files %s and %s differ\n" %
+                  (from_label, to_label)).encode('utf8'))

=== modified file 'bzrlib/inventory.py'
--- bzrlib/inventory.py	2007-10-24 20:38:50 +0000
+++ bzrlib/inventory.py	2007-11-21 14:55:46 +0000
@@ -619,25 +619,18 @@
     def _diff(self, text_diff, from_label, tree, to_label, to_entry, to_tree,
              output_to, reverse=False):
         """See InventoryEntry._diff."""
-        try:
-            from_text = tree.get_file(self.file_id).readlines()
-            if to_entry:
-                to_text = to_tree.get_file(to_entry.file_id).readlines()
-            else:
-                to_text = []
-            if not reverse:
-                text_diff(from_label, from_text,
-                          to_label, to_text, output_to)
-            else:
-                text_diff(to_label, to_text,
-                          from_label, from_text, output_to)
-        except errors.BinaryFile:
-            if reverse:
-                label_pair = (to_label, from_label)
-            else:
-                label_pair = (from_label, to_label)
-            output_to.write(
-                  ("Binary files %s and %s differ\n" % label_pair).encode('utf8'))
+        from bzrlib.diff import Differ
+        from_file_id = self.file_id
+        if to_entry:
+            to_file_id = to_entry.file_id
+        else:
+            to_file_id = None
+        if reverse:
+            to_file_id, from_file_id = from_file_id, to_file_id
+            tree, to_tree = to_tree, tree
+            from_label, to_label = to_label, from_label
+        differ = Differ(tree, to_tree, output_to, text_diff)
+        return differ.diff_text(from_file_id, to_file_id, from_label, to_label)
 
     def has_text(self):
         """See InventoryEntry.has_text."""
@@ -735,19 +728,21 @@
     def _diff(self, text_diff, from_label, tree, to_label, to_entry, to_tree,
              output_to, reverse=False):
         """See InventoryEntry._diff."""
-        from_text = self.symlink_target
+        from bzrlib.diff import Differ
+        old_target = self.symlink_target
         if to_entry is not None:
-            to_text = to_entry.symlink_target
-            if reverse:
-                temp = from_text
-                from_text = to_text
-                to_text = temp
-            output_to.write('=== target changed %r => %r\n' % (from_text, to_text))
-        else:
-            if not reverse:
-                output_to.write('=== target was %r\n' % self.symlink_target)
-            else:
-                output_to.write('=== target is %r\n' % self.symlink_target)
+            new_target = to_entry.symlink_target
+        else:
+            new_target = None
+        if not reverse:
+            old_tree = tree
+            new_tree = to_tree
+        else:
+            old_tree = to_tree
+            new_tree = tree
+            new_target, old_target = old_target, new_target
+        differ = Differ(old_tree, new_tree, output_to, text_diff)
+        return differ.diff_symlink(old_target, new_target)
 
     def __init__(self, file_id, name, parent_id):
         super(InventoryLink, self).__init__(file_id, name, parent_id)

